module Main where

import Codec.Encryption.Utils as U
import Codec.Encryption.Blowfish as Blowfish
import Codec.Encryption.AES as AES
import Codec.Encryption.Modes
import Codec.Encryption.Padding
import Codec.Encryption.LargeKey
import Codec.ASN1.ASN1 as A
import Data.Word
import Data.Bits
import Numeric
import Char
import Debug.QuickCheck

-- Blowfish Tests.

-- Tests from http://www.counterpane.com/vectors.txt.

d = 0xFEDCBA9876543210 :: Word64
k = 0xF0 :: Word8
e = 0xF9AD597C49DB005E :: Word64
e' = Blowfish.encrypt k d
blowfish1 = e == e'

k2 = 0xF0E1 :: Word16
e2 = 0xE91D21C1D961A6D6 :: Word64
e2' = Blowfish.encrypt k2 d
blowfish2 = e2 == e2'

e8 = 0xE87A244E2CC85E82 :: Word64
k8 = 0xF0E1D2C3B4A59687 :: Word64
e8' = Blowfish.encrypt k8 d
blowfish3 = e8 == e8'

-- Blowfish test with 128 bit key.

e16 = 0x93142887EE3BE15C :: Word64
k16= 0xF0E1D2C3B4A5968778695A4B3C2D1E0F :: Word128
e16' = Blowfish.encrypt k16 d
blowfish4 = e16 == e16'

-- Blowfish test with Cipher Block Chaining.
-- Set up the published key, initialization vector and data.

key16 = 0x0123456789ABCDEFF0E1D2C3B4A59687 :: Word128
iv8     = 0xFEDCBA9876543210 :: Word64
data29  = "7654321 Now is the time for \NUL"

-- Pad with nulls as in the example.

e29' = 
   cbc Blowfish.encrypt iv8 key16 $ padNulls $ map (fromIntegral . ord) data29

e29 = [0x6B,0x77,0xB4,0xD6,0x30,0x06,0xDE,0xE6,
       0x05,0xB1,0x56,0xE2,0x74,0x03,0x97,0x93,
       0x58,0xDE,0xB9,0xE7,0x15,0x46,0x16,0xD9,
       0x59,0xF1,0x65,0x2B,0xD5,0xFF,0x92,0xCC] :: [U.Octet]

blowfish5 = (concat $ map toOctets e29') == e29

instance Arbitrary Word8 where
   arbitrary = 
      do n <- choose ((fromIntegral (minBound::Word8))::Int, 
                      (fromIntegral (maxBound::Word8))::Int)
         return (fromIntegral n)

instance Arbitrary Word64 where
   arbitrary = 
      do n <- choose ((fromIntegral (minBound::Word64))::Integer, 
                      (fromIntegral (maxBound::Word64))::Integer)
         return (fromIntegral n)

instance Arbitrary Word128 where
   arbitrary = 
      do n <- choose ((fromIntegral (minBound::Word128))::Integer, 
                      (fromIntegral (maxBound::Word128))::Integer)
         return (fromIntegral n)

prop_decryptEncrypt k b = b == Blowfish.decrypt k (Blowfish.encrypt k b)
   where types = (k :: Word8, b :: Word64)

prop_AESIdempotent k b = b == AES.decrypt k (AES.encrypt k b)
   where types = (k :: Word128, b :: Word128)

prop_unCbcCbc iv k bs =
   bs == (unPkcs5 $ unCbc Blowfish.decrypt iv k $ cbc Blowfish.encrypt iv k $ pkcs5 bs)
      where types =(k :: Word8, iv :: Word64, bs :: [U.Octet])

prop_unPkcs5Pkcs5 os =
   os == (unPkcs5 $ ((pkcs5 os)::[Word64]))
      where types = (os :: [U.Octet])

prop_unNullsNulls os =
   all (/=0) os ==>
      os == (unPadNulls $ ((padNulls os)::[Word64]))
         where types = (os :: [U.Octet])

prop_unTwosCompTwosComp n =
   n < (0) ==>
      collect n $ 
      forAll g $ 
      \n -> n == (twosBinCompToInt $ intToTwosBinComp n)
      where types = (n::Int)
            g = do n <- choose (minBound::Int,maxBound::Int)
                   return (13*n)
